// Datei: I2CWRITE.C
//
// I2C-Bus Schreib/Leseroutinen
//
// hk@holger-klabunde.de 
// www.holger-klabunde.de 
// 23.03.2002
// Compiler CC5x
#include "prozess.h"      /* Der verwendete Prozessor */

#include "i2c.h" //Pindefinitionen
#include "protos.h"

unsigned char I2CERR;
unsigned char loop,mask; //Fr serielles Protokoll

//################################################
void I2C_STOP()
//################################################
 {  SCL_OFF(); SDA_OFF(); SCL_ON(); SDA_ON(); }

//################################################
void I2C_START()
//################################################
 {
   SCL_OFF(); SDA_OFF(); SCL_ON(); SDA_ON(); //I2C_STOP(); vorsichtshalber
   SDA_OFF(); SCL_OFF(); //Startbedingung
 }

//################################################
//RepeatedStart Condition
void I2C_REPSTART()
//################################################
 {
   SCL_OFF(); SDA_ON(); SCL_ON(); SDA_OFF(); SCL_OFF(); //Startbedingung
 }

//***************************
//Liest ein Byte
//ak=0 -> Weitere Bytes lesen (EEPROM)
//ak=1 -> Nur ein Byte lesen
//***************************
unsigned char I2C_Read(unsigned char ak)
{
 unsigned char by;

 by=0;  //Zum reinodern der Bits

 SDA_ON(); //Pullup SDA, auch fr ACK=1

 mask=0x80;

/* Byte lesen mit SCL 0,1,0 */
/* D7,D6,...D1,D0 */
 for(loop=0; loop<8; loop++)
  {
   SCL_ON();
   if(SDA==1) by|=mask; //1 Bits reinodern
   SCL_OFF();
   mask=mask>>1; //Ein Bit weiter rechts
  }

 if(ak==ACK) SDA_OFF(); //Fr ACK=0 -> Weitere Bytes lesen

 SCL_ON(); //ACK reintakten
 SCL_OFF();

 SDA_ON(); //Pullup SDA

 return by;
}

//***************************
//Schreibt ein Byte bitweise
//***************************
void I2C_Write(unsigned char by)
{
 mask=0x80;

/* Byte senden mit SCL 0,1,0 */
/* D7,D6,...D1,D0 */
 for(loop=0; loop<8; loop++)
  {
   if((by&mask)>0) { SDA_ON(); } /* eine 1 senden */
   else { SDA_OFF(); } /* sonst eine 0 */

   SCL_ON(); /* Flanke zum Daten bernehmen */
   SCL_OFF(); /* und fertig */
   mask=mask>>1;          /* nchstes Bit */
  }

/* Auf Acknowledge warten */
 SDA_ON(); /* Wird Eingang, Pullup zieht auf 1 */
 
 SCL_ON(); /* Ack raustakten */
 if(SDA==1)
  { I2CERR=1; } /* Kein ACK bei SDA=1 ! */
 else
  { I2CERR=0; } //Alles klar ACK=0

 SCL_OFF();

}
